﻿////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//
// DECLARACIÓN DE VARIABLES GENERALES DEL JUEGO

// Tamaño de los bloques del tablero y tamaño del arco de 2·PI radianes
var block = 15;      // cada bloque tiene un tamaño de 15x15 pixels
var pi2 = 2*Math.PI; // 2*PI radianes -> 360º

//SERPIENTES: 's1' y 's2', declaradas inicialmente como objetos vacíos
var s1 = {};
var s2 = {};

//MENSAJE FINAL AL TERMINAR EL JUEGO: vector (array) vacío
var MsjFinal = new Array();

// dimensiones [ancho,alto] del tablero de juego
var anchoZona=40;
var altoZona=24;

// frecuencia: determina la velocidad del juego (a menor frecuencia -> juego más rápido)
var frecuencia=80;

// idIntervalo: necesaria para finalizar el juego
var idIntervalo;

//COMIDA: declarada inicialmente como objeto vacío
var comida = {};

// Variable para el contexto (ctx) del canvas
var ctx;


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//
// DECLARACIÓN FUNCIONES DE PROPÓSITO GENERAL
// (serán llamadas desde otras funciones)

// Calcua los pixels de la coordenada 'n' (para dibujar cuadrados)
function blkRec(n) {return n*block;}

// Calcua los pixels de la coordenada 'n' (para dibujar círculos)
function blkCir(n) {return (n+0.5)*block;}

// Busca el elemento 'item' dentro de 'vector', devuelve la posición del elemento encontrado
function Buscar(vector, item, desde)
{
	var i, strItem = JSON.stringify(item);

	if(desde==undefined || !desde)
		desde=0;
	for(i=desde ; i<vector.length ; i++)
		if( JSON.stringify(vector[i]) == strItem )
			return i;
	return -1;
}


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//
// DECLARACIÓN FUNCIONES ESPECÍFICAS DEL JUEGO

////////////////////////////////////////////////////////////////////////////////
// Crea el vector de comida en coordenadas aleatorias
function CrearComida()
{
	var i, x, y;

	for(i=comida.elementos.length ; i<comida.total ; i++)
	{
		do
		{
			x=Math.floor((Math.random() * anchoZona));
			y=Math.floor((Math.random() * altoZona));
		}
		while( Buscar(comida.elementos, [x,y])>=0 || Buscar(s1.cuerpo, [x,y])>=0 || Buscar(s2.cuerpo, [x,y])>=0 );
		comida.elementos.push([x,y]);
	}
}

////////////////////////////////////////////////////////////////////////////////
// Dibuja los elementos del vector de comida en el tablero de juego
function DibujarComida()
{
	var i;

	for(i=0 ; i<comida.elementos.length ; i++)
	{
		ctx.beginPath();
		ctx.fillStyle=comida.color;
		ctx.strokeStyle=comida.color;
		ctx.arc(blkCir(comida.elementos[i][0]), blkCir(comida.elementos[i][1]) , block/2+3, 0, pi2);
		ctx.fill();
		ctx.stroke();
	}
}

////////////////////////////////////////////////////////////////////////////////
// Mover y dibujar una serpiente en el tablero de juego
function MoverSerpiente(sx)
{
	var c=sx.cuerpo;
	var cabeza=[ c[0][0]+sx.dirx , c[0][1]+sx.diry ];
	var idItem=Buscar(comida.elementos, cabeza);
	c.unshift(cabeza);

	// verificar si 'cabeza' alcanza un elemento de comida
	if(idItem>=0)
	{
		document.getElementById("mordisco").play();
		comida.elementos.splice(idItem, 1);
		CrearComida();
	}
	else
		c.pop();
}

function DibujarSerpiente(sx)
{
	var i, x, y, c=sx.cuerpo;

	// Cabeza: Circulo un poco más gordo
	ctx.beginPath();
	ctx.fillStyle=sx.color;
	ctx.strokeStyle=sx.color;
	ctx.arc(blkCir(c[0][0]), blkCir(c[0][1]) , block/2, 0, pi2);
	ctx.fill();
	ctx.stroke();

	// Cola: dos círculos consecutivos por cada posición
	for(i=1 ; i<c.length ; i++)
	{
		x=c[i][0];
		y=c[i][1];
		ctx.beginPath();
		ctx.fillStyle=sx.color;
		ctx.arc(blkCir(x), blkCir(y) , block/2-2, 0, pi2);

		x+=(c[i-1][0]-x)/2;
		y+=(c[i-1][1]-y)/2;
		ctx.arc(blkCir(x), blkCir(y) , block/2-2, 0, pi2);
		ctx.fill();
		ctx.stroke();
	}
}

////////////////////////////////////////////////////////////////////////////////
// Dibuja la explosión cuando ocurre colisión
function OndaExpansiva(x, y, r)
{
	var color=(r%2 ? "#ff0000" : "#ffff00");
	ctx.beginPath();
	ctx.strokeStyle=color;
	ctx.arc(blkCir(x), blkCir(y) , r, 0, pi2);
	ctx.stroke();
	if(r<block)
		setTimeout(function(){OndaExpansiva(x, y, r+1)}, 20);
}

////////////////////////////////////////////////////////////////////////////////
// Dibuja posición inicial de las serpientes en el CANVAS
function Empezar()
{
	// Estado inicial de las serpientes
	s1={
		"cuerpo": [[7,5], [6,5], [5,5], [4,5]],
		"color": "#ff8800", // NARANJA
		"dirx":1,
		"diry":0
	}
	s2={
		"cuerpo": [[32,18], [33,18], [34,18], [35,18]],
		"color": "#2060ff", // AZUL
		"dirx":-1,
		"diry":0
	}

	// comida
	comida={
		"elementos": [],   // vector (array) vacío
		"total": 2,        // número total de elementos de comida
		"color": "#ff0000" //
	};

	// Mensajes de fin de juego
	MsjFinal=[""
			, "Ganador <b style='color:"+s2.color+";'>'Serpiente Azul'</b>"
			, "Ganador <b style='color:"+s1.color+";'>'Serpiente Naranja'</b>"
			, "Empate ;-)"];

	ctx=document.getElementById("can").getContext("2d");
	ctx.clearRect(0, 0, anchoZona*block, altoZona*block);

	DibujarSerpiente(s1);
	DibujarSerpiente(s2);
	CrearComida();
	DibujarComida();
}

////////////////////////////////////////////////////////////////////////////////
// 5 segundos de cuenta atras y empezamos
function CuentaAtras(seg)
{
	if(seg<0)
	{
		// STAR
		document.getElementById("info").innerHTML="";
		idIntervalo=setInterval(function(){ESCENA()}, frecuencia);
	}
	else
	{
		document.getElementById("info").innerHTML=seg;
		setTimeout(function(){CuentaAtras(seg-1)}, 1000);
	}
}

////////////////////////////////////////////////////////////////////////////////
// Inicializa el juego
function Jugar()
{
	var i, seg=3;

	document.getElementById("inicio").currentTime=0;
	document.getElementById("inicio").play();
	document.getElementById("botonJugar").style.visibility="hidden";
	Empezar();
	CuentaAtras(seg);
}

////////////////////////////////////////////////////////////////////////////////
// Detecta si ha ocurido una colisión
function Colision()
{
	var x1, y1, x2, y2, i;
	var colS1=false;
	var colS2=false;

	// COMPROBAR s1
	x1=s1.cuerpo[0][0];
	y1=s1.cuerpo[0][1];
	// ¿ha chocado con los límites de la zona de juego?
	// ¿ha chocado con el cuerpo del otro jugador (s2)
	// ¿ha chocado con su propia cola?
	if(x1<0 || y1<0 || x1>=anchoZona || y1>=altoZona || Buscar(s2.cuerpo, [x1,y1])>=0 || Buscar(s1.cuerpo, [x1,y1], 1)>=1)
		colS1=true;

	// COMPROBAR s2
	x2=s2.cuerpo[0][0];
	y2=s2.cuerpo[0][1];
	// ¿ha chocado con los límites de la zona de juego?
	// ¿ha chocado con el cuerpo del otro jugador (s1)?
	// ¿ha chocado con su propia cola?
	if(x2<0 || y2<0 || x2>=anchoZona || y2>=altoZona || Buscar(s1.cuerpo, [x2,y2])>=0 || Buscar(s2.cuerpo, [x2,y2], 1)>=1)
		colS2=true;

	// SALIDA
	if(!colS1 && !colS2)
		return 0; // Sin colisión (Sigue el Juego)

	// sonido de impacto/fin del juego
	document.getElementById("fin").currentTime=0;
	document.getElementById("fin").play();

	if(colS1)
	{
        OndaExpansiva(x1, y1, 1);
		if(colS2)
		{
			if(x1!=x2 || y1!=y2)
				OndaExpansiva(x2, y2, 1);
			return 3; // han colisionado las 2 serpientes (empate)
		}
		return 1; // ha colisionado solo s1 (gana s2)
	}
	else
	{
		OndaExpansiva(x2, y2, 1);
		return 2; // ha colisionado solo s2 (gana s1)
	}
}

////////////////////////////////////////////////////////////////////////////////
// INTERVALO principal de juego, se repite indefinidamente hasta COLISIÓN
function ESCENA()
{
	MoverSerpiente(s1);
	MoverSerpiente(s2);

	var col = Colision();
	if( col > 0)
	{
		// FIN DEL JUEGO
		clearInterval(idIntervalo); // Cancelar el intervalo
		document.getElementById("info").innerHTML=MsjFinal[col]; // Mensaje resultado
		document.getElementById("botonJugar").style.visibility=""; // mostrar botón "jugar"
		return; //Terminar/Salir de la función
	}

	ctx.clearRect(0, 0, anchoZona*block, altoZona*block);

	document.getElementById("pasos").play();
	DibujarSerpiente(s1);
	DibujarSerpiente(s2);
	DibujarComida();
}

////////////////////////////////////////////////////////////////////////////////
// Modo de juego TECLADO / TÁCTIL
function CambioModoJuego()
{
	if(document.getElementById("BotonModoJuego").innerHTML.indexOf("Teclado")>=0)
	{
		document.getElementById("BotonModoJuego").innerHTML="Modo Juego<br/><big>[Táctil]</big>";
		document.getElementById("controlNaranja1").style.display="none";
		document.getElementById("controlNaranja2").style.display="";
		document.getElementById("controlAzul1").style.display="none";
		document.getElementById("controlAzul2").style.display="";
	}
	else
	{
		document.getElementById("BotonModoJuego").innerHTML="Modo Juego<br/><big>[Teclado]</big>";
		document.getElementById("controlNaranja1").style.display="";
		document.getElementById("controlNaranja2").style.display="none";
		document.getElementById("controlAzul1").style.display="";
		document.getElementById("controlAzul2").style.display="none";
	}
}

////////////////////////////////////////////////////////////////////////////////
// DETECTAR EVENTOS DEL TECLADO (modo de juego "TECLADO")
document.onkeydown=function(event)
{
	switch(event.keyCode)
	{
		////////////////////////////////////////////////////////////////////////
		// teclas para s1
		case 81: // Q=UP
			if(s1.diry==0)
			{
				s1.dirx=0;
				s1.diry=-1; // subir
			}
			break;
		case 65: // A=down
			if(s1.diry==0)
			{
				s1.dirx=0;
				s1.diry=1; // bajar
			}
			break;
		case 90: // Z=left
			if(s1.dirx==0)
			{
				s1.dirx=-1; // Izquierda
				s1.diry=0;
			}
			break;
		case 88: // X=right
			if(s1.dirx==0)
			{
				s1.dirx=1; // Derecha
				s1.diry=0;
			}
			break;

		////////////////////////////////////////////////////////////////////////
		// teclas para s2
		case 79: // O=UP
			if(s2.diry==0)
			{
				s2.dirx=0;
				s2.diry=-1; // subir
			}
			break;
		case 76: // L=down
			if(s2.diry==0)
			{
				s2.dirx=0;
				s2.diry=1; // bajar
			}
			break;
		case 188: // ,=left
			if(s2.dirx==0)
			{
				s2.dirx=-1; // Izquierda
				s2.diry=0;
			}
			break;
		case 190: // .=right
			if(s2.dirx==0)
			{
				s2.dirx=1; // Derecha
				s2.diry=0;
			}
			break;
	}
}

////////////////////////////////////////////////////////////////////////////////
// DETECTAR EVENTOS SOBRE BOTONES DE DIRECCIÓN (modo de juego "TÁCTIL")
function BotonesDireccion(x)
{
	switch(x)
	{
		////////////////////////////////////////////////////////////////////////
		// teclas para s1
		case "S1_UP": // Q=UP
			if(s1.diry==0)
			{
				s1.dirx=0;
				s1.diry=-1; // subir
			}
			break;
		case "S1_DOWN": // A=down
			if(s1.diry==0)
			{
				s1.dirx=0;
				s1.diry=1; // bajar
			}
			break;
		case "S1_LEFT": // Z=left
			if(s1.dirx==0)
			{
				s1.dirx=-1; // Izquierda
				s1.diry=0;
			}
			break;
		case "S1_RIGHT": // X=right
			if(s1.dirx==0)
			{
				s1.dirx=1; // Derecha
				s1.diry=0;
			}
			break;

		////////////////////////////////////////////////////////////////////////
		// teclas para s2
		case "S2_UP": // O=UP
			if(s2.diry==0)
			{
				s2.dirx=0;
				s2.diry=-1; // subir
			}
			break;
		case "S2_DOWN": // L=down
			if(s2.diry==0)
			{
				s2.dirx=0;
				s2.diry=1; // bajar
			}
			break;
		case "S2_LEFT": // ,=left
			if(s2.dirx==0)
			{
				s2.dirx=-1; // Izquierda
				s2.diry=0;
			}
			break;
		case "S2_RIGHT": // .=right
			if(s2.dirx==0)
			{
				s2.dirx=1; // Derecha
				s2.diry=0;
			}
			break;
	}
}